@@ -1,3 +1,4 @@
+import math
 import collections
 import copy
 from typing import List, Dict, OrderedDict
@@ -119,7 +120,7 @@ def sam_lcs1(sam: SuffixAutomaton, t: List[str], min_len: int):
     length = 0  # 当前
     longest = 0  # 全局
     cands = []  # 候选
-    for x in t:
+    for i, x in enumerate(t):
         if x in sam.nodes[p].next:  # 匹配
             p = sam.nodes[p].next[x]
             length += 1
@@ -135,17 +136,18 @@ def sam_lcs1(sam: SuffixAutomaton, t: List[str], min_len: int):
         if length > 0:
             longest = max(longest, length)
             endpos = p
-            cands.append((endpos, length))
+            cand_start = i
+            cands.append((endpos, length, cand_start))
     if min_len <= 0:
         min_len = max(1, longest)
     ans = []
-    for endpos, length in cands:
+    for endpos, length, cand_start in cands:
         if length >= min_len:
             (t, start) = sam.sub_seq(endpos, length)
             if ans and ans[-1][1] == start:
-                ans[-1] = (t, start)
+                ans[-1] = (t, start, cand_start)
             else:
-                ans.append((t, start))
+                ans.append((t, start, cand_start))
     return ans
 
 
@@ -218,7 +220,6 @@ def lcs2(query: List[str], doc: List[List[str]], min_len: int = -1):
     re = sam_lcs2(sam, doc, min_len)
     return re
 
-
 if __name__ == "__main__":
     # http://123.57.137.208/ccf/ccf-4.jsp
     raw = """
@@ -242,19 +243,19 @@ if __name__ == "__main__":
     # print(sam2)
     # print(sam_lcs1(sam1, s2))
 
-    # [(['Software', 'Engineering'], 14)]
+    # [(['Software', 'Engineering'], 14, 6)]
     print(lcs1(doc[1], doc[2]))
     # [([':'], 1), (['on'], 4), (['Software'], 6)]
     print(lcs2(doc[0], doc[1:4]))
 
-    # [([':'], 1), (['Conference'], 7), (['on'], 10), (['Software', 'Engineering'], 14)]
+    # [([':'], 1, 1), (['Conference'], 7, 3), (['on'], 10, 4), (['Software', 'Engineering'], 14, 6)]
     print(lcs1(doc[1], doc[2], 1))
     # [([':'], 1), (['on'], 4), (['Software'], 6)]
     print(lcs2(doc[0], doc[1:4], 1))
 
     poet = "江天一色无纤尘皎皎空中孤月轮 江畔何人初见月江月何年初照人 人生代代无穷已江月年年望相似 不知江月待何人但见长江送流水"
     doc = poet.split()
-    # [(['人'], 13), (['江'], 0), (['江', '月'], 7), (['年'], 10)]
-    print(lcs1(doc[1], doc[2], 1))
+    # [(['江'], 0, 2), (['江', '月'], 7, 3), (['何'], 9, 5), (['何', '人'], 2, 6), (['见'], 5, 8), (['江'], 0, 10)]
+    print(lcs1(doc[1], doc[3], 1))
     # [(['人'], 0), (['江', '月'], 7)]
     print(lcs2(doc[2], doc[2:4], 1))
